/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DualCount.h"

using namespace std;

namespace cadarache {

const string DualCount::TYPE = "dual_acquisition";

/*
 * Constructor
 */
DualCount::DualCount(const std::string& name) :
		KineticCount(name) {

	registerFunction(TYPE);

	m_DPPTimeGate.init(this, "dpp_gate");
}

/*
 * postConfiguration
 */
void DualCount::postConfiguration() {
	Count::postConfiguration();

	m_KineticAcquisitionControllers.resize(nDetectors());
	for (int32 i = 0; i < nDetectors(); ++i) {
		m_KineticAcquisitionControllers[0].copy(acquisitionControllers[0]);
	}
	registerUpdater(m_KineticAcquisitionControllers[0]->time, &DualCount::updateActualTime, this);
	registerUpdater(m_KineticAcquisitionControllers[0]->timeSetPoint, &DualCount::updateTimeSetPoint, this);
	registerUpdater(m_KineticAcquisitionControllers[0]->nbRepaint, &DualCount::updateNbRepaint, this);
}

/*
 * preStart
 */
void DualCount::preStart() {
	m_KineticAcquisitionControllers[0]->slicesFile = slicesFile();
	m_KineticAcquisitionControllers[0]->nbRepaint.setpoint = localReps();

}

/*
 * gateStart
 */
void DualCount::gateStart() {
	m_DPPTimeGate->mode = acquisition::TimeAcquisitionController::TIME_MODE;
	m_DPPTimeGate->timeType = acquisition::TimeAcquisitionController::SECOND;
	m_DPPTimeGate->time.setpoint = m_TimeSetPointSecond;
	time10.setpoint = m_TimeSetPointSecond * 10;
	time100.setpoint = m_TimeSetPointSecond * 100;
	time1000.setpoint = m_TimeSetPointSecond * 1000;
	preset = m_TimeSetPointSecond * 1000.;
	m_KineticAcquisitionControllers[0]->startCountingParallel(false);
	// wait kinetic ready
	while(m_KineticAcquisitionControllers[0]->gateStatus() != acquisition::GateCommon::WAITFORSTART_STATUS) {
		boost::this_thread::yield();
	}
	m_DPPTimeGate->startCountingCommand();
}

/*
 * gateSynchro
 */
void DualCount::gateSynchro() {
	// Start gate
	//m_KineticGate->synchronizeReadCommand(false);
	m_DPPTimeGate->synchronizeReadCommand(false);
}

/*
 * pause
 */
void DualCount::pause() {
	log(Level::s_Info) << "pause" << endlog;
	m_KineticAcquisitionControllers[0]->pauseParallel(false);
	m_DPPTimeGate->pauseParallel(false);
}

/*
 * resume
 */
void DualCount::resume() {
	m_KineticAcquisitionControllers[0]->resumeParallel(false);
	m_DPPTimeGate->resumeParallel(false);
	log(Level::s_Info) << "continu" << endlog;
}

/*
 * stop
 */
void DualCount::stop() {

	m_DPPTimeGate->stopParallel(false);
	m_KineticAcquisitionControllers[0]->stopParallel(false);

	log(Level::s_Info) << "stopped" << endlog;

}

}
