/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CAENPSDCONTROLLER_H
#define CAENPSDCONTROLLER_H

#include "drivers/caen/caen_psd/CAENPsdDriver.h"
#include "CAENAdcController.h"

namespace npp {

/*!
 * \class CAENV1724Driver
 * \brief Main class for the CAENV1724Driver device driver
 *
 * \par
 * Class for CAENV1724Driver driver. Implements DeviceDriver methods.
 * \par
 * The CAENV1724Driver device manages the global functionality of the CAEN V1724 VME Digitiser.
 */


class CAENPsdController: public CAENAdcController {

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	CAENPsdController(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~CAENPsdController();

	/**
	 * \brief Postconfiguration
	 */
	virtual void postConfiguration();

	// Channels properties
    Property<float64>triggerHoldOff;
    Property<int32>triggerThreshold;
    Property<int32>selfTrigger;
    Property<int32>chargeSensitivity;
    Property<float64>shortGate;
    Property<float64>longGate;
    Property<float64>preGate;
    Property<int32>triggerValidationWindow;
    Property<int32>baselineMean;
	void initBoard();

protected:

	DriverPtr<caen_psd::CAENPsdDriver> m_dgtzDriver;

	virtual void refreshNbChannelsProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshDisplayChannelProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshDppAcqModeProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshPulsePolarityProperty(int32 aValue) throw (CannotSetValue);
	virtual void updateBoardProperties();

private:

	virtual void refreshChannelActiveProperty(int32 aValue) throw (CannotSetValue);

	virtual void refreshRecordLengthProperty(int32 aValue) throw (CannotSetValue);
	void refreshTriggerHoldOffProperty(float64 aValue) throw (CannotSetValue);
	void refreshTriggerThresholdProperty(int32 aValue) throw (CannotSetValue);
	void refreshSelfTriggerProperty(int32 aValue) throw (CannotSetValue);
	void refreshChargeSensitivityProperty(int32 aValue) throw (CannotSetValue);
	void refreshShortGateProperty(float64 aValue) throw (CannotSetValue);
	void refreshLongGateProperty(float64 aValue) throw (CannotSetValue);
	void refreshPreGateProperty(float64 aValue) throw (CannotSetValue);
	void refreshBaselineMean(int32 aValue) throw (CannotSetValue);
	void refreshDcOffsetProperty(int32 aValue) throw (CannotSetValue);
	void refreshPreTriggerProperty(int32 aValue) throw (CannotSetValue);

	virtual void clearData();
	virtual void writeParam();
	virtual void initChannel(int32 aChannel);
};

}
#endif //CAENV1724CONTROLLER_H
