/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DIELECTRICS_REMOTEWINDETA_H
#define DIELECTRICS_REMOTEWINDETA_H

#include <Controller.h>
#include <cameo/cameo.h>
#include "controllers/common/utilities/CountSpy.h"
#include "drivers/utilities/null/NullDriver.h"
#include <thread>

namespace dielectrics {

/*!
 * \brief Test command.
 */
class Test : private controller::Command {

public:
	Test(ExperimentController * c) :
		controller::Command(c, "test", &Test::test, this) {
	}

	virtual void test() = 0;
	void testCommand(bool logging = false) {
		command(logging, true);
	}
};

class RemoteWindeta : public ExperimentController,
									public controller::Stoppable,
									public controller::Init,
									public controller::Raz,
									public Test {

public:
	//! Type of controller
	static const std::string TYPE;

	RemoteWindeta(const std::string& name);
	RemoteWindeta(const RemoteWindeta& controller);
	virtual ~RemoteWindeta();

	virtual void postConfiguration();

	virtual void setEnabled(bool all, bool postconf = true);
	virtual void setDisabled(bool all = false);

	virtual void init();
	virtual void start();
	virtual void stop();
	virtual void raz();
	virtual void test();

	Property<std::string> serverEndpoint;
	Property<bool> initialized;
	Property<int32> applicationState;

	Property<std::string> preset;
	Property<int32> numberOfPoints;
	Property<int32> startTimeout;

	Property<std::string> dataFilenameFormat;
	Property<int32> dataFilenameFormatSize;
	DynamicProperty<std::string> dataFilenameFormatValues;
	DynamicProperty<std::string> dataFilenameFormatLabels;

	Property<int32> presetsChanged;
	Property<int32> presetsSize;
	DynamicProperty<std::string> presets;

private:
	void reset();
	bool initApplication();
	void stopApplication();
	void subscriberLoop();

	void updateStatusMessage();
	void initPresets();
	void initIndependentDataFileId();
	void setDataFilename();
	void saveData(const std::string& data);

	static const std::string WINDETA_APPLICATION;
	static const std::string RESPONDER_NAME;
	static const std::string PUBLISHER_NAME;
	static const std::string INDEPENDENT_PREFIX;

	static const std::string INDEPENDENT_VALUE;
	static const std::string NUMOR_VALUE;
	static const std::string NUMOR_1_VALUE;
	static const std::string COUNT_SYNC_VALUE;

	std::unique_ptr<cameo::Server> m_server;
	std::unique_ptr<cameo::application::Instance> m_windetaApplication;
	std::unique_ptr<cameo::application::Requester> m_requester;
	std::unique_ptr<cameo::application::Subscriber> m_subscriber;
	std::unique_ptr<std::thread> m_subscriberThread;

	ControllerPtr<utilities::CountSpy> m_CountSpy;
	DriverPtr<driver::NullDriver> m_driver;

	int m_independentDataFileId;
	int m_countNumor;
	bool m_stopped;
	std::string m_dataFilename;
};

}

#endif
