/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef SIMULATEDCADARACHEDRIVER_H
#define SIMULATEDCADARACHEDRIVER_H

#include <drivers/ioxos/cadaracheacq/CadaracheAcqState.h>
#include "Utilities/Counter.h"
#include <vector>

namespace cadaracheacq {

/*!
 * \class SimulatedCadaracheAcqDriver
 * \brief Simulated implementation class for the Cadarache device driver
 *
 * This class is a perfect implementation of StartStop device driver.
 * On start command, all actual values become target's ones.
 */
class SimulatedCadaracheAcqDriver: public CadaracheAcqState {

public:

	/*!
	 * \brief Constructor
	 * \param[in] owner The device driver main class link
	 */
	SimulatedCadaracheAcqDriver(CadaracheAcqDriver* owner);

	/*!
	 * \brief Destructor
	 */
	virtual ~SimulatedCadaracheAcqDriver();


	/*!
	 * \brief Init command implementation
	 */
	virtual void init();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readStatus();

	/*!
	 * \brief Start command implementation
	 */
	virtual void start();

	/*!
	 * \brief Read command implementation
	 */
	virtual void read();

	/*!
	 * \brief Synchronize Read command implementation
	 */
	virtual void synchroniseRead();

	/*!
	 * \brief Stop command implementation
	 */
	virtual void stop();

	/*!
	 * \brief Pause command implementation
	 */
	virtual void pause();

	/*!
	 * \brief Resume command implementation
	 */
	virtual void resume();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readInfos();

	/*!
	 * \brief Write para command implementation
	 */
	virtual void writeParam();

	/*!
	 * \brief Read par command implementation
	 */
	virtual void readParam();

	/*!
	 * \brief Start command implementation
	 */
	virtual void clear();

private:

	void timer();
	bool m_Pause; // ! Pause flag
	bool m_Stop; // ! Pause flag
	int32 m_Status; // Local status memory
	Counter m_TimeCounter; //! Time counter
	int32 m_ActualSlice;
	std::vector<float64> m_Times;
	bool m_Finished;
};

}
#endif //SIMULATEDCADARACHEDRIVER_H
