/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef STEREOFLUXGATECONTROLLER_H
#define STEREOFLUXGATECONTROLLER_H

#include <Controller.h>
#include "StereoFluxgateDriver.h"

namespace stereo_fluxgate {


/*!
 * \class StereoFluxgateController
 */

class StereoFluxgateController: public ExperimentController{

public:

	//! Type of controller
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 *
	 * It is the default constructor called by the creator class ExperimentControllerCreator.
	 *
	 * \param[in] name the name of the controller
	 */
	StereoFluxgateController(const std::string& name);

	/*!
	 * \brief Copy constructor
	 *
	 * \param[in] controller the controller to copy
	 */
	StereoFluxgateController(const StereoFluxgateController& controller);

	/*!
	 * \brief Destructor
	 */
	virtual ~StereoFluxgateController();

	/*!
	 * \brief Method called before changing the property value
	 *
	 * This method is called after setting configuration during the creation of controller.
	 */
	virtual void postConfiguration();

	/*!
	 * Properties
	 */


	Property<int32> nbfluxgate;
	DynamicProperty<int32> ichannel;
	DynamicProperty<float64> Bx;
	DynamicProperty<float64> By;
	DynamicProperty<float64> Bz;
	DynamicProperty<float64> Btotal;


protected:


	DriverPtr<StereoFluxgateDriver> m_Driver; //! The driver linked to the controller




private:
	void refreshNbFluxgateProperty(int32 value) throw (CannotSetValue);
	virtual void updateNbFluxgate();
	virtual void updateBx(int32 index);
	virtual void updateBy(int32 index);
	virtual void updateBz(int32 index);

	/*!
	 * \brief update the progression property from driver one
	 */
	void updateProgression();
};
}
#endif // STEREOFLUXGATECONTROLLER_H
